/*
 * say_ru.c
 */

/*---   sound files:

	enumeration:
	------------
	h-N0		()
	h-hundredG	(sotiy(-oe,-aya))
	h-thousandG	(tisyachniy(-oe,-aya))
	h-millionG	(millionniy(-oe,-aya))
	h-Mh		(odno-, dvuh-, tryoh-, ...)
	h-Y0h		(desyati-, dvadtsati-, ...)
	h-Y00h		(sto-, dvuhsot-, ...)
	h-MF		(f gender)
	h-MN		(n gender)
	
	    where N from 2 to 9
	    where M from 1 to 19
	    where Y from 1 to 9
	    where G in M,N,F

	number:
	-------
	n00			(one hundred, two hundred, ...) where 'n' from 1 to 9
	1F			(odna)
	1N			(odno)
	2F			(dve)
	thousand
	thousands-i
	thousands
	million
	million-a
	millions
	billion
	billion-a
	billions
	
	date/time:
	----------
	hour			(chas)
	hours-a			(chasa)
	hours			(chasov)
	minute			(minuta)
	minutes-i		(minuty)
	minutes			(minut)
	second			(sekunda)
	seconds-i		(sekundy)
	seconds			(sekund)
	
*/

char* ru_num_phrases[3][3] = {{"digits/thousand","digits/thousands-i","digits/thousands"},
			      {"digits/million","digits/million-a","digits/millions"},
			      {"digits/billion","digits/billion-a","digits/billions"}};

char* ru_time_phrases[3][3] = {{"digits/hour","digits/hours-a","digits/hours"},
			      {"digits/minute","digits/minutes-i","digits/minutes"},
			      {"digits/second","digits/seconds-i","digits/seconds"}};

static int ru_enum_part(const say_args_t *a, int num)
{
    int res = 0;
    int h = num / ONE_HUNDRED;
    int t = num % ONE_HUNDRED;
    int d = num % 10;
    
    if (t < 10)
		t = 0;
    else if (t < 20)
		d = 0;
    else
		t = t - d;
	if (h) {
		res = s_streamwait(a, "digits/h-%d00h", h);
		if (res)
			return res;
    }
    if (t) {
		res = s_streamwait(a, "digits/h-%dh", t);
		if (res)
			return res;
    }
    if (d) {
		res = s_streamwait(a, "digits/h-%dh", d);
		if (res)
			return res;
    }
    return res;
}

/* Helper functions, that need for make code shorter */
/*--- determine what case of number name we must used to (ru) */
static int ru_get_index(const int n)
{
	if (n % ONE_HUNDRED >= 11 && n % ONE_HUNDRED <= 14)
	    return 2;
	switch (n % 10) {
	case 1:
		return 0;
	case 2:
	case 3:
	case 4:
		return 1;
	default:
		return 2;
	}
}

/*--- say_number_full_ru: Russian syntax */
static int say_number_full_ru(const say_args_t *a, int num, const char *options)
{
	int res = 0;
	int digits = 0;
	char *gender;
	
	if (!num) 
		return say_digits_full(a, 0);
	
	if (options && !strncasecmp(options, "f",1))
		gender = "F";
	else if (options && !strncasecmp(options, "n",1))
		gender = "N";
	else
		gender = "";

	while (!res && (num)) {
		char fn[256] = "";

		if (num < 0) {
			snprintf(fn, sizeof(fn), "digits/minus");
			num = negate(num);
		} else	if (num < 20) {
			snprintf(fn, sizeof(fn), "digits/%d%s", num, gender);
			num = 0;
		} else	if (num < ONE_HUNDRED) {
			snprintf(fn, sizeof(fn), "digits/%d", num - (num % 10));
			num %= 10;
		} else 	if (num < ONE_THOUSAND){
			snprintf(fn, sizeof(fn), "digits/%d", num - (num % ONE_HUNDRED));
			num %= ONE_HUNDRED;
		} else 	if (num < ONE_MILLION) { /* 1,000,000 */
			digits = num / ONE_THOUSAND;
			/* say thousands */
			if (digits % ONE_HUNDRED < 3)
				res = say_number_full_ru(a, digits, "f");
			else
				res = say_number_full_ru(a, digits, NULL);
			if (res)
				return res;
			snprintf(fn, sizeof(fn), ru_num_phrases[0][ru_get_index(digits)]);
			num %= ONE_THOUSAND;
		} else 	if (num < ONE_BILLION) {	/* 1,000,000,000 */
			digits = num / ONE_MILLION;
			/* say millions */
			res = say_number_full_ru(a, digits, NULL);
			if (res)
				return res;
			snprintf(fn, sizeof(fn), ru_num_phrases[1][ru_get_index(digits)]);
			num %= ONE_MILLION;
		} else 	if (num < INT_MAX) {	/* 1,000,000,000 and bigger */
			digits = num / ONE_BILLION;
			/* say billions */
			res = say_number_full_ru(a, digits, NULL);
			if (res)
				return res;
			snprintf(fn, sizeof(fn), ru_num_phrases[2][ru_get_index(digits)]);
			num %= ONE_BILLION;
		} else {
			ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
				res = -1;
		}
		if (!res)
			res = s_streamwait3(a, fn);
	}
	return res;
}

/*--- say_enumeration_full_ru: Russian syntax */
static int say_enumeration_full_ru(const say_args_t *a, int num, const char *options)
{
	int res = 0;
	char fn[256] = "", *gender;
	int tens, thousands, millions;

	if (options && !strncasecmp(options, "f",1))
		gender = "F";
	else if (options && !strncasecmp(options, "n",1))
		gender = "N";
	else
		gender = "";

	if (!num) 
		return say_digits_full(a, 0);
	while (!res && num) {
		if (num < 0) {
			snprintf(fn, sizeof(fn), "digits/minus"); /* kind of senseless for enumerations, but our best effort for error checking */
			num = negate(num);
		} else if (num < 20) {
			snprintf(fn, sizeof(fn), "digits/h-%d%s", num, gender);
			num = 0;
		} else if (num < ONE_HUNDRED) { 
			tens = num / 10;
			num %= 10;
			if (num == 0)
				snprintf(fn, sizeof(fn), "digits/h-%d%s", (tens * 10), gender);
			else
				snprintf(fn, sizeof(fn), "digits/%d", (tens * 10)); /* XXX no gender here ? */
		} else if (num < ONE_THOUSAND) {
			if (num % ONE_HUNDRED) {	/* say 100, 200, ... */
				res = say_number_full_ru(a, (num / ONE_HUNDRED) * ONE_HUNDRED, options);
				if (res)
					return res;
				num %= ONE_HUNDRED;
			} else {	/* say 100th, 200th, ... */
				if (num / ONE_HUNDRED == 1) { /* 100th */
					snprintf(fn, sizeof(fn), "digits/h-hundred%s", gender);
					num = 0;
				} else { /* 2x-, 3x-, ... */
					snprintf(fn, sizeof(fn), "digits/h-%dh", num / ONE_HUNDRED);
					num = 100;
				}
			}
		} else if (num < ONE_MILLION) { /* 1,000,000 */
			thousands = num / ONE_THOUSAND;
			if (num % ONE_THOUSAND) { /* number of thousands */
				res = say_number_full_ru(a, thousands * ONE_THOUSAND, options);
				if (res)
					return res;
			} else {
				if (thousands != 1) {
					res = ru_enum_part(a,thousands); 
					if (res)
					    return res;
				}
				snprintf(fn, sizeof(fn), "digits/h-thousand%s", gender);
			}
			num %= ONE_THOUSAND;
		} else if (num < ONE_BILLION) {
			millions = num / ONE_MILLION;
			if (num % ONE_MILLION) {
				res = say_number_full_ru(a, millions * ONE_MILLION, options);
				if (res)
					return res;
			} else {
				res = ru_enum_part(a,millions);
				if (res)
				    return res;
				snprintf(fn, sizeof(fn), "digits/h-million%s", gender);
			}
			num %= ONE_MILLION;
		} else if (num == INT_MAX) {
			snprintf(fn, sizeof(fn), "digits/h-last");
			num = 0; 
		} else {
			ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
			res = -1;
		}
		if (!res)
			res = s_streamwait3(a, fn);
	}
	return res;
}

/* Russian syntax */
static int say_date_with_format_ru(const say_args_t *a, const struct tm *tm1,  const char *format, time_t delta)
{
	struct tm tm = *tm1;
	int res=0, offset;
	char c;
	say_date_with_format_f *f = say_date_with_format_ru; /* shorthand for recursive call */

	for (offset=0; res == 0 && (c = format[offset]) != '\0'; offset++) {
		char nextmsg[256] = "";
		switch (c) {
		case 'm':
			/* Month enumerated */
			res = say_enumeration_full_ru(a, (tm.tm_mon + 1), NULL);
			break;
		case 'd':
		case 'e':
			/* First - Thirtyfirst */
			res = say_enumeration_full_ru(a, tm.tm_mday, "n");	
			break;
		case 'Y':
			/* Year */
			res = say_enumeration_full_ru(a, 1900 + tm.tm_year, "m");
			break;
		case 'I':
		case 'l':
			/* 12-Hour */
			if (tm.tm_hour == 0) {
				/* do we really need to say 12 instead of 0 ? */
				snprintf(nextmsg, sizeof(nextmsg), "digits/12");
			} else if (tm.tm_hour > 12) {
				snprintf(nextmsg, sizeof(nextmsg), "digits/%d", tm.tm_hour - 12);
			} else {
				snprintf(nextmsg, sizeof(nextmsg), "digits/%d", tm.tm_hour);
			}
			res = s_streamwait3(a, nextmsg);
			if (!res)
				res = s_streamwait3(a, ru_time_phrases[0][ru_get_index(tm.tm_hour)]);
			break;
		case 'H':
		case 'k':
			/* 24-Hour */
			res = say_number_full_ru(a, tm.tm_hour, NULL);
			if (!res)
				res = s_streamwait3(a, ru_time_phrases[0][ru_get_index(tm.tm_hour)]);
			break;
		case 'M':
		case 'N':
			/* Minute */
			res = say_number_full_ru(a, tm.tm_min, NULL);
			if (!res)
				res = s_streamwait3(a, ru_time_phrases[1][ru_get_index(tm.tm_min)]);
			break;
		case 'P':
		case 'p':
			/* AM/PM */
			if (tm.tm_hour >= 12)
				snprintf(nextmsg, sizeof(nextmsg), "digits/p-m");
			else
				snprintf(nextmsg, sizeof(nextmsg), "digits/a-m");
			res = s_streamwait3(a, nextmsg);
			break;
		case 'S':
			/* Seconds */
			res = say_number_full_ru(a, tm.tm_sec, NULL);
			if (!res)
				res = s_streamwait3(a, ru_time_phrases[2][ru_get_index(tm.tm_sec)]);
			break;
                case 'c': /* national representation of time and date */
                        res = f(a, &tm, "xX", delta);
                        break;
                case 'X': /* national representation of time */
                        res = f(a, &tm, "HM", delta);
                        break;
                case 'x': /* national representation of date */
                        res = f(a, &tm, "ABdY", delta);
                        break;
		default: /* common part of date format */
			res = say_date_common(a, &tm, format, delta, &offset, f);
		}
	}
	return res;
}

static struct _lang_sw map_ru = {
        .language = "ru",
        .say_number_full = say_number_full_ru,
        .say_enumeration_full = say_enumeration_full_ru,
        .say_date_with_format = say_date_with_format_ru,
        .date_fmt = "x",
        .time_fmt = "X",
        .datetime_fmt = "c",
};
